from .base_api import BaseApi 
from sweetspot.https.api_response import ApiResponse 
from sweetspot.api_helper import ApiHelper 
from sweetspot.https.auth.o_auth_2 import OAuth2

class ShopApi(BaseApi):

    def __init__(self, config):
        super().__init__(config)

    def get_all_shops(self):
        _url_path = f'/shop'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result
               

    def get_shop(self, shop_id):
        _url_path = f'/shop/{shop_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers) 
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        
        return _result
    
    def update_shop(self, shop_id, payload): 
        _url_path = f'/shop/{shop_id}' 
        _query_builder = self.config.get_base_uri() 
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
                'accept': 'application/json',
                'Content-Type': 'application/json', 
        }

        _request = self.config.http_client.patch(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        
        return _result

    def replace_shop(self, shop_id, payload): 
        _url_path = f'/shop/{shop_id}' 
        _query_builder = self.config.get_base_uri() 
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
                'accept': 'application/json',
                'Content-Type': 'application/json', 
        }

        _request = self.config.http_client.put(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        
        return _result

    def add_shop(self, payload):
        _url_path = f'/shop'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path
        _query_url = ApiHelper.clean_url(_query_builder)
        
        _headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json; charset=utf-8'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def get_categories(self, shop_id, page=None, size=None, sort_by=None, sort=None):
        _url_path = f'/shop/{shop_id}/category'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_parameters = {
            'page': page, 
            'size': size,
            'sort_by': sort_by, 
            'sort': sort
        }
        _query_builder = ApiHelper.append_url_with_query_parameters(_query_builder, _query_parameters)
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result
    
    def get_items(self, shop_id, page=None, size=None, sort_by=None, sort=None):
        _url_path = f'/shop/{shop_id}/item'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_parameters = {
            'page': page, 
            'size': size,
            'sort_by': sort_by, 
            'sort': sort
        }
        _query_builder = ApiHelper.append_url_with_query_parameters(_query_builder, _query_parameters)
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def get_modifiers(self, shop_id, page=None, size=None, sort_by=None, sort=None):
        _url_path = f'/shop/{shop_id}/modifier'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'page': page, 
            'size': size,
            'sort_by': sort_by, 
            'sort': sort
        }
        _query_builder = ApiHelper.append_url_with_query_parameters(_query_builder, _query_parameters) 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def get_menus(self, shop_id, page=None, size=None, sort_by=None, sort=None):
        _url_path = f'/shop/{shop_id}/menu'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_parameters = {
            'page': page, 
            'size': size,
            'sort_by': sort_by, 
            'sort': sort
        }
        _query_builder = ApiHelper.append_url_with_query_parameters(_query_builder, _query_parameters)
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def delete_shop(self, shop_id):
        _url_path = f'/shop/{shop_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.delete(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request) 
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        
        return _result

    def add_hour(self, payload):
        _url_path = f'/shop/hour'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def get_hour(self, hour_id):
        _url_path = f'/shop/hour/{hour_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def update_hour(self, hour_id, payload):
        _url_path = f'/shop/hour/{hour_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json', 
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.patch(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def replace_hour(self, hour_id, payload):
        _url_path = f'/shop/hour/{hour_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json', 
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.put(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def delete_hour(self, hour_id):
        _url_path = f'/shop/hour/{hour_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.delete(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def add_multiple_hours(self, payload):
        _url_path = f'/shop/hour/multiple'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json', 
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def delete_multiple_hours(self, payload):
        _url_path = f'/shop/hour/multiple'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json', 
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.delete(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result


    



from flask.views import MethodView 
from flask import jsonify, make_response, request 
from flask_jwt_extended import jwt_required

class TestGeneralApiEndpoints(MethodView):

    @jwt_required()
    def get(self):
        data = {'success':True}
        resp = make_response(data, 200)
        return resp 

    @jwt_required()
    def post(self):
        data = request.get_json() 
        assert data.get('serialized') == True 
        data = {'success':True}
        resp = make_response(data, 201)
        return resp 

class TestSpecificApiEndpoints(MethodView):

    def get(self, id):
        data = {'success': True}
        resp = make_response(data, 200)
        return resp 

    @jwt_required()
    def patch(self, id):
        data = request.get_json()
        assert data.get('serialized') == True
        data = {'success': True}
        resp = make_response(data, 201)
        return resp 

    @jwt_required()
    def put(self, id):
        data = request.get_json()
        assert data.get('serialized') == True
        data = {'success': True}
        resp = make_response(data, 201)
        return resp  

    @jwt_required()
    def delete(self, id):
        resp = make_response('', 204)
        return resp 


import requests
import os

from .http_response import HttpResponse
from .http_request import HttpRequest 

class HttpClient: 

    def __init__(self):
        self.session = requests.session()

    def execute(self, request):
        response = self.session.request(
            request.http_method,
            request.query_url, 
            headers=request.headers,
            params=request.query_parameters,
            data=request.parameters, 
            cookies=request.cookies,
            files=request.files)

        return self.convert_response(response, request)

    def convert_response(self, response, http_request):
        return HttpResponse(response.status_code,
                            response.reason, 
                            response.headers,
                            response.text, 
                            response.cookies, 
                            http_request)
    
    def get(self, query_url, 
            query_parameters={},
            headers={},
            parameters={},
            cookies={},
            files={}):

        return HttpRequest(
            "GET", 
            query_url,
            headers,
            query_parameters, 
            parameters, 
            cookies,
            files)

    def post(self, query_url, 
            query_parameters={},
            headers={},
            parameters={},
            cookies={},
            files={}):

        return HttpRequest(
            "POST",
            query_url, 
            headers,
            query_parameters, 
            parameters, 
            cookies,
            files)

    def put(self, query_url, 
            headers={},
            query_parameters={},
            parameters={},
            cookies={},
            files={}):

        return HttpRequest(
            "PUT",
            query_url, 
            headers,
            query_parameters, 
            parameters, 
            cookies,
            files)

    def patch(self, query_url, 
            headers={},
            query_parameters={},
            parameters={},
            cookies={},
            files={}):

        return HttpRequest(
            "PATCH",
            query_url, 
            headers,
            query_parameters, 
            parameters, 
            cookies,
            files)

    def delete(self, query_url, 
            query_parameters={},
            headers={},
            parameters={},
            cookies={},
            files={}):

        return HttpRequest(
            "DELETE", 
            query_url,
            headers,
            query_parameters, 
            parameters, 
            cookies,
            files)


from .base_api import BaseApi 
from sweetspot.https.api_response import ApiResponse 
from sweetspot.api_helper import ApiHelper 
from sweetspot.https.auth.o_auth_2 import OAuth2

class NotificationApi(BaseApi):

    def __init__(self, config):
        super().__init__(config)

    def get_shop_notifications(self, shop_id):
        _url_path = f'/notification/shop/{shop_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result


import pytest

from multiprocessing import Process
from flask import request, make_response
from flask.views import MethodView
from flask_jwt_extended import jwt_required
from .endpoints import TestGeneralApiEndpoints, TestSpecificApiEndpoints
from sweetspot.client import Client

@pytest.fixture(scope='module')
def server(app):
    class TestMerchantApiEndpoints(MethodView):

        @jwt_required()
        def get(self):
            data = {'success':True}
            resp = make_response(data, 200)
            return resp 

        def post(self):
            data = request.get_json()
            assert data.get('serialized') == True 
            data = {'success':True}
            resp = make_response(data, 201)
            return resp

    app.add_url_rule('/api/merchant', view_func=TestMerchantApiEndpoints.as_view('general'))
    app.add_url_rule('/api/merchant/<uuid:id>', view_func=TestSpecificApiEndpoints.as_view('specific'))

    server = Process(target=app.run)
    
    yield server.start()

    server.terminate()
    server.join()

def test_get_all_merchants(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.merchant.get_all_merchants()
    assert result.is_success() == True 

def test_add_merchant(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.merchant.add_merchant(payload)
    assert result.is_success() == True 

def test_get_merchant(server, sample_uuid):
    client = Client(environment='test')
    result = client.merchant.get_merchant(sample_uuid)
    assert result.is_success() == True 

def test_update_merchant(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.merchant.update_merchant(sample_uuid, payload)
    assert result.is_success() == True 

def test_replace_merchant(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.merchant.replace_merchant(sample_uuid, payload)
    assert result.is_success() == True 

def test_delete_merchant(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.merchant.delete_merchant(sample_uuid)
    assert result.is_success() == True




import pytest

from flask import Flask, jsonify, make_response, request, Blueprint
from flask.views import MethodView
from flask_jwt_extended import JWTManager, set_refresh_cookies, set_access_cookies, create_access_token, create_refresh_token, get_csrf_token

from sweetspot.client import Client

@pytest.fixture(scope='module')
def app():
    app = Flask(__name__)
    app.config['JWT_SECRET_KEY'] = 'secretkey'
    app.config['JWT_TOKEN_LOCATION'] = ['cookies']
    JWTManager(app)

    yield app

@pytest.fixture 
def sample_uuid():
    return '59cb4747-0761-4612-9783-43c91ce14594'

@pytest.fixture(scope='function')
def sample_cookie_resp():
    access_token = create_access_token('test')
    refresh_token =  create_refresh_token('refresh')
    sample_cookie_resp = {
        'access_token_cookie': access_token,
        'refresh_token_cookie': refresh_token, 
        'csrf_access_token': get_csrf_token(access_token),
        'csrf_refresh_token': get_csrf_token(refresh_token)
    }
    return sample_cookie_resp





    




import pytest
import os 
import requests

from multiprocessing import Process
from flask import request, make_response
from flask.views import MethodView
from flask_jwt_extended import jwt_required
from .endpoints import TestGeneralApiEndpoints, TestSpecificApiEndpoints
from sweetspot.client import Client

@pytest.fixture(scope='module')
def server(app):

    class ItemPictureUpload(MethodView):

        @jwt_required()
        def post(self, id):
            if 'image' not in request.files: 
                resp = make_response({'success':False}, 400)
            image = request.files.get('image')
            data = {'success':True}
            resp = make_response(data, 201)
            return resp

    app.add_url_rule('/api/item', view_func=TestGeneralApiEndpoints.as_view('general'))
    app.add_url_rule('/api/item/<uuid:id>', view_func=TestSpecificApiEndpoints.as_view('specific'))
    app.add_url_rule('/api/item/picture/<uuid:id>', view_func=ItemPictureUpload.as_view('upload'))

    server = Process(target=app.run)
    
    yield server.start()

    server.terminate()
    server.join()

def test_get_all_items(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.item.get_all_items()
    assert result.is_success() == True 

def test_add_item(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.item.add_item(payload)
    assert result.is_success() == True 

def test_get_item(server, sample_uuid):
    client = Client(environment='test')
    result = client.item.get_item(sample_uuid)
    assert result.is_success() == True 

def test_update_item(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.item.update_item(sample_uuid, payload)
    assert result.is_success() == True 

def test_replace_item(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.item.replace_item(sample_uuid, payload)
    assert result.is_success() == True 

def test_delete_item(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.item.delete_item(sample_uuid)
    assert result.is_success() == True

def test_upload_picture(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    image = open(os.getcwd() + '/tests/static/no-photo-available.png', 'rb')
    result = client.item.upload_picture(sample_uuid, ('no-photo-available.png', image, 'image/png'))
    assert result.is_success() == True 
    





import pytest
import requests
import json

from multiprocessing import Process
from flask import request, make_response
from flask.views import MethodView
from flask_jwt_extended import jwt_required
from .endpoints import TestGeneralApiEndpoints, TestSpecificApiEndpoints
from sweetspot.client import Client

@pytest.fixture(scope='module')
def server(app):

    class TestUnprotectedPostEndpoint(MethodView):

        def post(self):
            resp = make_response('', 201)
            return resp

    class TestUnprotectedPostEndpointWithID(MethodView):

        def post(self, id):
            resp = make_response('', 201)
            return resp

    class TestProtectedPostEndpoint(MethodView):

        @jwt_required()
        def post(self):
            resp = make_response('', 201)
            return resp

    class TestProtectedRefreshPostEndpoint(MethodView):

        @jwt_required(refresh=True)
        def post(self):
            resp = make_response('', 201)
            return resp

    app.add_url_rule('/api/authentication/change-password', view_func=TestProtectedPostEndpoint.as_view('change_password'))
    app.add_url_rule('/api/authentication/refresh-token', view_func=TestProtectedRefreshPostEndpoint.as_view('refresh'))
    app.add_url_rule('/api/authentication/verify-email-token', view_func=TestProtectedPostEndpoint.as_view('verify_email_token'))
    app.add_url_rule('/api/authentication/verify-email/<uuid:id>', view_func=TestUnprotectedPostEndpointWithID.as_view('verify_merchant_email'))
    app.add_url_rule('/api/authentication/login', view_func=TestUnprotectedPostEndpoint.as_view('login'))
    app.add_url_rule('/api/authentication/logout', view_func=TestUnprotectedPostEndpoint.as_view('logout'))


    server = Process(target=app.run)
    
    yield server.start()

    server.terminate()
    server.join()

def test_change_password(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.authentication.change_password(payload)
    assert result.is_success() == True 

def test_refresh_token(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.authentication.refresh_access()
    assert result.is_success() == True 

def test_verify_email_token(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.authentication.verify_email_token(payload)
    assert result.is_success() == True 

def test_send_verification_email(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.authentication.send_verification_email(sample_uuid)
    assert result.is_success() == True 

def test_login(server):
    client = Client(environment='test')
    payload = {'serialized':True}
    result = client.authentication.get_authentication_information(payload)
    assert result.is_success() == True 

def test_logout(server):
    client = Client(environment='test')
    result = client.authentication.unset_access()
    assert result.is_success() == True



class cached_property(object):
    def __init__(self, func, name=None):
        self.func = func
        self.name = name if name is not None else func.__name__
        self.__doc__ = func.__doc__

    def __get__(self, instance, class_):
        if instance is None:
            return self
        res = self.func(instance)
        setattr(instance, self.name, res)
        return res



class OAuth2: 

    @staticmethod 
    def apply(config, http_request):
        csrf_access_token = config.csrf_access_token 
        access_token_cookie = config.access_token_cookie 
        http_request.headers['X-CSRF-TOKEN'] = csrf_access_token 
        http_request.cookies['access_token_cookie'] = access_token_cookie

    @staticmethod 
    def apply_refresh(config, http_request):
        csrf_refresh_token = config.csrf_refresh_token
        refresh_token_cookie = config.refresh_token_cookie 
        http_request.headers['X-CSRF-TOKEN'] = csrf_refresh_token 
        http_request.cookies['refresh_token_cookie'] = refresh_token_cookie

        
         




class HttpResponse(object):

    def __init__(self, 
                status_code, 
                reason_phrase, 
                headers, 
                text, 
                cookies, 
                request):

        self.status_code = status_code 
        self.reason_phrase = reason_phrase 
        self.headers = headers 
        self.text = text 
        self.cookies = cookies.get_dict()
        self.request = request 
        


from copy import deepcopy

from sweetspot.https.http_client import HttpClient

class Configuration(object): 

	environments = {
		'development': 'http://merchant-api:5000/api/merchant',
		'production': 'https://sweetspot.so/api/merchant',
		'test': 'http://localhost:5000/api/merchant'
	}

	def __init__(self, csrf_access_token=None, 
				 csrf_refresh_token=None, 
				 access_token_cookie=None, 
				 refresh_token_cookie=None,
				 environment='development'):
		self.csrf_access_token = csrf_access_token
		self.csrf_refresh_token = csrf_refresh_token
		self.access_token_cookie = access_token_cookie 
		self.refresh_token_cookie = refresh_token_cookie
		self.environment = environment
		self.http_client = self.create_http_client()

	def get_base_uri(self):
		return self.environments[self.environment]

	def create_http_client(self):
		return HttpClient()




from .base_api import BaseApi 
from sweetspot.https.api_response import ApiResponse 
from sweetspot.api_helper import ApiHelper 
from sweetspot.https.auth.o_auth_2 import OAuth2

class CategoryApi(BaseApi):

    def __init__(self, config):
        super().__init__(config)

    def get_all_categories(self):
        _url_path = f'/category'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers) 
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def get_category(self, category_id):
        _url_path = f'/category/{category_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict:
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def update_category(self, category_id, payload):
        _url_path = f'/category/{category_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.patch(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict:
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def replace_category(self, category_id, payload):
        _url_path = f'/category/{category_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.put(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict:
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def delete_category(self, category_id): 
        _url_path = f'/category/{category_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.delete(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict:
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result


    def add_category(self, payload):
        _url_path = f'/category'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path
        _query_url = ApiHelper.clean_url(_query_builder)
        
        _headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else: 
            _errors = None
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def add_item(self, category_id, item_id):
        _url_path = f'/category/{category_id}/item/{item_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else: 
            _errors = None
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def delete_item(self, category_id, item_id):
        _url_path = f'/category/{category_id}/item/{item_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.delete(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else: 
            _errors = None
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 


        

    

    



import jsonpickle 
import re

from requests.utils import quote

class ApiHelper(object):

    @staticmethod
    def merge_dicts(dict1, dict2):
        temp = dict1.copy()
        temp.update(dict2)
        return temp

    @staticmethod
    def clean_url(url):
        regex = "^http?://[^/]+"
        match = re.match(regex, url)
        if match is None:
            raise ValueError('Invalid Url format.')

        protocol = match.group(0)
        index = url.find('?')
        query_url = url[len(protocol): index if index != -1 else None]
        query_url = re.sub("//+", "/", query_url)
        parameters = url[index:] if index != -1 else ""

        return protocol + query_url + parameters

    @staticmethod 
    def json_serialize(obj):
        return jsonpickle.encode(obj)

    @staticmethod 
    def json_deserialize(json):
        if not json: 
            return json 
        try:
            decoded = jsonpickle.decode(json)
            return decoded 
        except ValueError: 
            return json 

    @staticmethod
    def append_url_with_query_parameters(url,
                                        parameters,
                                        array_serialization="indexed"):
        if url is None:
            raise ValueError("URL is None.")
        if parameters is None:
            return url

        for key, value in parameters.items():
            seperator = '&' if '?' in url else '?'
            if value is not None:
                if isinstance(value, list):
                    value = [element for element in value if element]
                    if array_serialization == "csv":
                        url += "{0}{1}={2}".format(
                            seperator,
                            key,
                            ",".join(quote(str(x), safe='') for x in value)
                        )
                    elif array_serialization == "psv":
                        url += "{0}{1}={2}".format(
                            seperator,
                            key,
                            "|".join(quote(str(x), safe='') for x in value)
                        )
                    elif array_serialization == "tsv":
                        url += "{0}{1}={2}".format(
                            seperator,
                            key,
                            "\t".join(quote(str(x), safe='') for x in value)
                        )
                    else:
                        url += "{0}{1}".format(
                            seperator,
                            "&".join(("{0}={1}".format(k, quote(str(v), safe='')))
                                        for k, v in APIHelper.serialize_array(key, value, array_serialization))
                        )
                else:
                    url += "{0}{1}={2}".format(seperator, key, quote(str(value), safe=''))

        return url



         



class HttpRequest: 

    def __init__(self, 
                http_method, 
                query_url, 
                headers=None, 
                query_parameters=None,
                parameters=None, 
                cookies=None,  
                files=None):
                
        self.http_method = http_method 
        self.query_url = query_url 
        self.headers = headers 
        self.query_parameters = query_parameters 
        self.parameters = parameters 
        self.cookies = cookies
        self.files = files 



import json


class ApiResponse:

    def __init__(self, http_response,
                 body=None,
                 errors=None):

        self.status_code = http_response.status_code
        self.reason_phrase = http_response.reason_phrase
        self.headers = http_response.headers
        self.text = http_response.text
        self.request = http_response.request
        self.cookies = http_response.cookies
        self.body = body
        self.errors = errors 

    @property 
    def errors(self):
        return self._errors 

    @errors.setter
    def errors(self, errors):
        if type(errors) in [dict, list]: 
            errors = self._parse_errors(errors)
        self._errors = errors

    def is_success(self):
        return 200 <= self.status_code < 300

    def is_error(self):
        return 400 <= self.status_code < 600

    def _parse_errors(self, errors):
        for key in errors: 
            curr = errors.get(key)
            if isinstance(curr, dict):
                return self._parse_errors(curr)
            elif isinstance(curr, list):
                return curr[0]
            else:
                return curr

    def __repr__(self):
        return f'<ApiResponse [{self.text}]>'


import pytest

from multiprocessing import Process
from flask import request, jsonify
from flask.views import MethodView
from flask_jwt_extended import jwt_required
from .endpoints import TestSpecificApiEndpoints, TestGeneralApiEndpoints
from sweetspot.client import Client

@pytest.fixture(scope='module')
def server(app):
    
    app.add_url_rule('/api/shop', view_func=TestGeneralApiEndpoints.as_view('general'))
    app.add_url_rule('/api/shop/<uuid:id>', view_func=TestSpecificApiEndpoints.as_view('specific'))
    app.add_url_rule('/api/shop/<uuid:id>/category', view_func=TestSpecificApiEndpoints.as_view('category'))
    app.add_url_rule('/api/shop/<uuid:id>/item', view_func=TestSpecificApiEndpoints.as_view('item'))
    app.add_url_rule('/api/shop/<uuid:id>/modifier', view_func=TestSpecificApiEndpoints.as_view('modifier'))
    app.add_url_rule('/api/shop/<uuid:id>/option', view_func=TestSpecificApiEndpoints.as_view('option'))

    server = Process(target=app.run)

    yield server.start()

    server.terminate()
    server.join()

def test_get_all_shops(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.shop.get_all_shops()
    assert result.is_success() == True 

def test_add_shop(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.shop.add_shop(payload)
    assert result.is_success() == True 

def test_get_shop(server, sample_uuid):
    client = Client(environment='test')
    result = client.shop.get_shop(sample_uuid)
    assert result.is_success() == True 

def test_update_shop(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.shop.update_shop(sample_uuid, payload)
    assert result.is_success() == True 

def test_replace_shop(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.shop.replace_shop(sample_uuid, payload)
    assert result.is_success() == True 

def test_delete_shop(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.shop.delete_shop(sample_uuid)
    assert result.is_success() == True

def test_get_categories(server, sample_uuid):
    client = Client(environment='test')
    result = client.shop.get_categories(sample_uuid)
    assert result.is_success() == True 

def test_get_items(server, sample_uuid):
    client = Client(environment='test')
    result = client.shop.get_items(sample_uuid)
    assert result.is_success() == True 

def test_get_modifiers(server, sample_uuid):
    client = Client(environment='test')
    result = client.shop.get_modifiers(sample_uuid)
    assert result.is_success() == True 

def test_get_options(server, sample_uuid):
    client = Client(environment='test')
    result = client.shop.get_options(sample_uuid)
    assert result.is_success() == True







from functools import lru_cache

from .configuration import Configuration
from .decorators import cached_property
from .apis.beta_user_api import BetaUserApi
from .apis.authentication_api import AuthenticationApi
from .apis.merchant_api import MerchantApi
from .apis.shop_api import ShopApi
from .apis.option_api import OptionApi 
from .apis.item_api import ItemApi
from .apis.modifier_api import ModifierApi
from .apis.category_api import CategoryApi
from .apis.notification_api import NotificationApi
from .apis.square_api import SquareApi

class Client: 

    def __init__(self, csrf_access_token=None, 
                 csrf_refresh_token=None, 
                 access_token_cookie=None, 
                 refresh_token_cookie=None,
                 environment='development', 
                 config=None):
        if config: 
            self.config = config 
        else:
            self.config = Configuration(
                csrf_access_token=csrf_access_token, 
                csrf_refresh_token=csrf_refresh_token,
                access_token_cookie=access_token_cookie, 
                refresh_token_cookie=refresh_token_cookie,
                environment=environment)

    def update_config(self, **kwargs):
        for k, v in kwargs.items():
            if hasattr(self.config, k):
                setattr(self.config, k, v)
                
    @cached_property
    def beta_user(self):
    	return BetaUserApi(self.config)

    @cached_property 
    def authentication(self):
        return AuthenticationApi(self.config)

    @cached_property 
    def merchant(self):
        return MerchantApi(self.config)

    @cached_property 
    def shop(self):
        return ShopApi(self.config)

    @cached_property 
    def category(self):
        return CategoryApi(self.config)

    @cached_property 
    def modifier(self):
        return ModifierApi(self.config)

    @cached_property 
    def item(self):
        return ItemApi(self.config)

    @cached_property 
    def option(self):
        return OptionApi(self.config)

    @cached_property
    def notification(self):
        return NotificationApi(self.config)

    @cached_property 
    def square(self):
        return SquareApi(self.config)


                





class BaseApi(object):

    def __init__(self, config):
        self._config = config 

    @property 
    def config(self):
        return self._config 

    def validate_parameters(self, **kwargs):
        for name, value in kwargs.items():
            if not value: 
                raise ValueError(f"Required parameter {name} cannot be None")

    def execute_request(self, request):
        func = self.config.http_client.execute
        response = func(request)

        return response 

        



from .base_api import BaseApi 
from sweetspot.https.api_response import ApiResponse 
from sweetspot.api_helper import ApiHelper 
from sweetspot.https.auth.o_auth_2 import OAuth2


class BetaUserApi(BaseApi):

    def __init__(self, config):
        super().__init__(config)

    def add_beta_user(self, body):
        _url_path = '/beta-user'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json', 
            'Content-Type': 'application/json; charset=utf-8'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(body))
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def get_beta_user(self, id):
        _url_path = f'/beta-user/{id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def get_all_beta_users(self):
        _url_path = f'/beta-user'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json'
        }

        _request = self.config.http_client.get(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 


        




import pytest

from multiprocessing import Process
from flask import request
from .endpoints import TestGeneralApiEndpoints, TestSpecificApiEndpoints
from sweetspot.client import Client

@pytest.fixture(scope='module')
def server(app):
    app.add_url_rule('/api/modifier', view_func=TestGeneralApiEndpoints.as_view('general'))
    app.add_url_rule('/api/modifier/<uuid:id>', view_func=TestSpecificApiEndpoints.as_view('specific'))

    server = Process(target=app.run)
    
    yield server.start()

    server.terminate()
    server.join()

def test_get_all_modifiers(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.modifier.get_all_modifiers()
    assert result.is_success() == True 

def test_add_modifier(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.modifier.add_modifier(payload)
    assert result.is_success() == True 

def test_get_modifier(server, sample_uuid):
    client = Client(environment='test')
    result = client.modifier.get_modifier(sample_uuid)
    assert result.is_success() == True 

def test_update_modifier(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.modifier.update_modifier(sample_uuid, payload)
    assert result.is_success() == True 

def test_replace_modifier(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    payload = {'serialized':True}
    result = client.modifier.replace_modifier(sample_uuid, payload)
    assert result.is_success() == True 

def test_delete_modifier(server, sample_cookie_resp, sample_uuid):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.modifier.delete_modifier(sample_uuid)
    assert result.is_success() == True




import pytest 

from multiprocessing import Process
from flask import request, make_response
from flask.views import MethodView
from .endpoints import TestGeneralApiEndpoints, TestSpecificApiEndpoints
from sweetspot.client import Client

@pytest.fixture(scope='module')
def server(app):

    class TestUnprotectedPostEndpoint(TestGeneralApiEndpoints):

        def post(self):
            data = request.get_json()
            assert data.get('serialized') == True 
            data = {'success':True}
            resp = make_response(data, 201)
            return resp 

    app.add_url_rule('/api/beta-user', view_func=TestUnprotectedPostEndpoint.as_view('general'))
    app.add_url_rule('/api/beta-user/<uuid:id>', view_func=TestSpecificApiEndpoints.as_view('specific'))

    server = Process(target=app.run)
    
    yield server.start()

    server.terminate()
    server.join()

def test_add_beta_user(server):
    client = Client(environment='test')
    payload = {'serialized':True}
    result = client.beta_user.add_beta_user(payload)
    assert result.is_success() == True 

def test_get_beta_user(server, sample_uuid):
    client = Client(environment='test')
    result = client.beta_user.get_beta_user(sample_uuid)
    assert result.is_success() == True 

def test_get_all_beta_users(server, sample_cookie_resp):
    client = Client(environment='test')
    client.update_config(**sample_cookie_resp)
    result = client.beta_user.get_all_beta_users()
    assert result.is_success() == True


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="sweetspot-sdk",
    version="0.1.5",
    description="SDK for Sweetspot Team to interact with backend APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Philip Kung",
    author_email="philip@sweetspot.so",
    url="https://www.sweetspot.so",
    packages=find_packages(),
    install_requires=[
        'requests>=2.9.1, <3.0',
        'jsonpickle>=1.4, >= 1.4.1',
        'python-dateutil>=2.5.3, <3.0',
        'deprecation>=2.0.6'
    ]
)



from .base_api import BaseApi 
from sweetspot.https.api_response import ApiResponse 
from sweetspot.api_helper import ApiHelper 
from sweetspot.https.auth.o_auth_2 import OAuth2

class AuthenticationApi(BaseApi):

    def __init__(self, config):
        super().__init__(config)

    def get_authentication_information(self, payload):
        _url_path = '/authentication/login'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json; charset=utf-8'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def refresh_access(self):
        _url_path = '/authentication/refresh-token'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers)
        OAuth2.apply_refresh(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict:   
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def unset_access(self):
        _url_path = '/authentication/logout'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers) 
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 
        
        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def send_verification_email(self, merchant_id):
        _url_path = f'/authentication/verify-email/{merchant_id}'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers)
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text) 
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

    def verify_email_token(self, payload):
        _url_path = f'/authentication/verify-email-token'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = { 
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }    

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)  

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result 

    def change_password(self, payload):
        _url_path = f'/authentication/change-password'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path 
        _query_url = ApiHelper.clean_url(_query_builder)

        _headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }

        _request = self.config.http_client.post(_query_url, headers=_headers, parameters=ApiHelper.json_serialize(payload))
        OAuth2.apply(self.config, _request)
        _response = self.execute_request(_request)

        decoded = ApiHelper.json_deserialize(_response.text)
        if type(decoded) == dict: 
            _errors = decoded.get('error')
        else:
            _errors = None 

        _result = ApiResponse(_response, body=decoded, errors=_errors)
        return _result

        